<?php
/**
 * @file
 * Chunks API. Define functions to interact with chunks.
 *
 * Provide other modules with useful functions.
 */

/**
 * chunks_all().
 * Get all avaiable chunks. Will search all chunks,
 * that are altered by other modules.
 *
 * @return array(), all avaiable chunks.
 */
function chunks_all() {

  // allow all modules to alter the avaiable chunks.
  $chunks = array();
  drupal_alter('chunks', $chunks);
  return $chunks;
}

/**
 * chunks_load_all().
 * Get all avaiable chunks and load their classes.
 *
 * @return array(), all avaiable chunks.
 */
function chunks_load_all($perm = NULL) {
  $chunks = chunks_all();

  // for all chunks
  foreach ($chunks as $key => $chunk) {

    // if chunk is not avaiable
    if (!chunks_load_chunk($chunk)) {
      unset($chunks[$key]);
    }

    // if user don't have permissions to access chunk
    if ($perm != NULL && !user_access($perm . '_' . $key)) {
      unset($chunks[$key]);
    }
  }

  // return all chunks.
  return $chunks;
}

/**
 * chunks_load_chunk($chunk).
 * @param $chunk, a chunk-array.
 * @return TRUE, if file was loaded.
 */
function chunks_load_chunk($chunk) {

  if (
    !isset($chunk['name']) ||
    !isset($chunk['module']) ||
    !isset($chunk['class'])
  ) {
    return FALSE;
  }

  // include chunk class file if present.
  $file = DRUPAL_ROOT . '/' . drupal_get_path('module', $chunk['module']);
  $file .= '/' . $chunk['name'];
  if (is_file($file)) {
    require_once $file;

    // check if the class exists
    if (class_exists($chunk['class'], FALSE)) {
      return TRUE;
    }
  }

  return FALSE;
}

/**
 * chunks_get($key).
 *
 * @param $key, get the chunk with this key.
 * @return the found Chunk klass, or null, if no chunk was found.
 *
 * Will search all chunks, that are included by other modules.
 */
function chunks_get($key) {

  // get all chunks
  $chunks = chunks_all();

  // if the requested chunk is avaiable
  if (array_key_exists($key, $chunks)) {
    return $chunks[$key];
  }

  return NULL;
}

/**
 * chunks_get_and_load($key).
 *
 * @param $key, get the chunk with this key.
 * @return TRUE, if file was loaded.
 *
 * Will search all chunks, that are included by other modules and load the file.
 */
function chunks_load_by_key($key) {
  if (($chunk = chunks_get($key)) != NULL) {
    if (chunks_load_chunk($chunk)) {
      return $chunk;
    }

    else {
      return NULL;
    }
  }
}

/**
 * chunks_validate_configuration($chunk, $configuration).
 * Test if a chunk configuration is valid.
 *
 * @param $key, String, the key of of the chunk to test
 * @param $configuration, array(), the configuration array.
 * @return TRUE, if the configuration is valid for this chunk, otherwise FALSE.
 */
function chunks_validate_configuration($key, $configuration) {

  // try to get the chunk-class
  $chunk = chunks_load_by_key($key);
  if ($chunk != NULL) {

    // init chunk and return validation result.
    $chunk_instance = new $chunk['class']($configuration);
    return $chunk_instance->validate();
  }

  else {

    // return false, if chunk was not found.
    return FALSE;
  }
}

/**
 * chunks_view($chunk, $configuration).
 * Creates a renderable array for the chunk.
 *
 * @param $key, String, the key of of the chunk to render.
 * @param $configuration, array(), configuration for the chunk.
 * @return renderable array, or null if chunk was not found.
 */
function chunks_view($key, $configuration) {

  // try to get the chunk-class
  $chunk = chunks_load_by_key($key);
  if ($chunk != NULL) {

    // init chunk and return renderable view.
    $chunk_instance = new $chunk['class']($configuration);
    return $chunk_instance->get_renderable_view();

  }

  else {
    // return null, if chunk was not found.
    return NULL;
  }
}
