<?php
/**
 * @file
 * Chunks chunk. The base class to all chunks.
 */

interface ChunksChunkInterface {
  /**
   * title().
   * Define the readable title of this chunk
   *
   * @return String, chunk's title
   */
  static function title();

  /**
   * description().
   * Describe functionality and configuration options.
   *
   * @return String, chunk's title
   */
  static function description();

  /**
   * configuration().
   * Define any configuration options for this chunk.
   *
   * @return array(), configuration definition.
   */
  static function configuration();

  /**
   * image().
   * Define an image for this chunk.
   *
   * @return String, path to image
   */
  static function image();
}

abstract class ChunksChunk implements ChunksChunkInterface {

  /**
   * holds any configuration for this chunk, if avaiable.
   */
  protected $configuration = array();

  /**
   * holds a renderable view. Each chunk must use the query function to fill
   * this array with renderable elements.
   */
  protected $view = array();

  /**
   * __construct($configuration).
   * @param $configuration, array(), configuration for this chunk.
   */
  function __construct($configuration) {
    $this->configuration = $configuration;
  }

  /**
   * get_renderable_view().
   * returns a renderable array of this chunk.
   * @return array(), a renderable array for this chunk, or null if
   * configuration is not valide.
   */
  final function get_renderable_view() {

    // validate configuraiton
    if ($this->validate()) {

      //call the query method
      $this->query();

      // return the view array of this chunk.
      return $this->view;

    }

    else {
      return NULL;
    }
  }

  /**
   * query().
   * Does the entity querying stuff for this chunk. Query is also the place to
   * fill $this->view with renderable elements.
   *
   * get_renderable_view() will call query() right before returning $this->view.
   */
  protected abstract function query();

  /**
   * Default implementation of configuration(): There is no configuration.
   */
  static public function configuration() {
    // the default is: no configuration.
    return NULL;
  }

  /**
   * Default implementation of image(): There is no image.
   */
  static public function image() {
    // the default is: no image.
    return NULL;
  }

  /**
   * validate_configuration($configuration).
   * @return false, if there is any invalid configuration, otherwise true.
   */
  public function validate() {
    return TRUE;
  }
}
